bl_info = {
    "name" : "Sketching Idea",
    "author" : "Samuli Riihikoski (haikalle)",
    "description" : "Sketcing ideas with grease pencil.",
    "blender" : (2, 80, 0),
    "version": (0, 1, 0, 2),
    "location" : "",
    "warning" : "",
    "category" : "Generic"
}

if "bpy" in locals():
    import importlib

import bpy
import bmesh
import mathutils

from . draw_op import OT_draw_operator
from bpy.types import PropertyGroup, Menu
from bpy.props import (
        BoolProperty,
        EnumProperty,
        IntProperty,
        FloatProperty,
        StringProperty,
        PointerProperty,
        )

addon_keymaps = []

class ShowHide(bpy.types.Operator):
    bl_idname = "object.showhide"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):
        grease_pencil = bpy.context.scene.objects['SketchingIdea'].data
        hide_all = bpy.context.scene.sketchIdea.hide_all

        if(hide_all):
            for layer in grease_pencil.layers:
                layer.hide = False

        else:
            for layer in grease_pencil.layers:
                layer.hide = True

        if(hide_all):
            bpy.context.scene.sketchIdea.hide_all = False
        else:
            bpy.context.scene.sketchIdea.hide_all = True

        return {'FINISHED'}

class DeleteAll(bpy.types.Operator):
    bl_idname = "object.delete_all"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):
        active_object = bpy.context.view_layer.objects.active
        active_mode = bpy.context.view_layer.objects.active.mode

        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.context.view_layer.objects.active = bpy.context.view_layer.objects['SketchingIdea']
        bpy.ops.object.mode_set(mode='PAINT_GPENCIL')


        grease_pencil = bpy.context.scene.objects['SketchingIdea']

        for layer in grease_pencil.data.layers:
            grease_pencil.data.layers.remove(layer)


        nimi = "SketchingIdea"
        sketchIdea_mesh = bpy.data.meshes[nimi]

        bpy.data.meshes.remove(sketchIdea_mesh)

        bpy.data.meshes.new(name=nimi)
        bpy.data.meshes[nimi].use_fake_user = True

        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.context.view_layer.objects.active = active_object
        bpy.ops.object.mode_set(mode=active_mode)


        return {'FINISHED'}

class RotateStrokes(bpy.types.Operator):
    bl_idname = "object.rotate_strokes"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):
        ghost = bpy.context.scene.sketchIdea.ghost

        grease_pencil = bpy.context.scene.objects['SketchingIdea'].data
        grease_rotate = bpy.context.scene.objects['Rotate_Ghost']

        print('ghost:', ghost)
        if(ghost == False):
            for index, layer in enumerate(bpy.data.meshes["SketchingIdea"].vertices):
                if (layer.co[1] < -480):
                    grease_pencil.layers[index].parent = grease_rotate

            grease_rotate.rotation_euler[2] = 1.5708
            bpy.context.scene.sketchIdea.ghost = True

        if(ghost == True):
            for index, layer in enumerate(bpy.data.meshes["SketchingIdea"].vertices):
                    grease_pencil.layers[index].parent = None
            bpy.context.scene.sketchIdea.ghost = False
            grease_rotate.rotation_euler[2] = 0




        return {'FINISHED'}

class DeleteVisible(bpy.types.Operator):
    bl_idname = "object.delete_visible"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):

        active_object = bpy.context.view_layer.objects.active
        active_mode = bpy.context.view_layer.objects.active.mode

        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.context.view_layer.objects.active = bpy.context.view_layer.objects['SketchingIdea']
        bpy.ops.object.mode_set(mode='PAINT_GPENCIL')

        lista = []
        grease_pencil = bpy.context.scene.objects['SketchingIdea'].data
        nimi = "SketchingIdea"
        me = bpy.data.meshes[nimi]

        delete_list = []
        layer_count = len(grease_pencil.layers)

        for index, layer in enumerate(bpy.data.meshes[nimi].vertices):
            if (grease_pencil.layers[index].opacity > 0.3 and grease_pencil.layers[index].parent == None):
                delete_list.append(grease_pencil.layers[index].info)
            else:
                lista.append((layer.co[0], layer.co[1], layer.co[2]))

        for delete in delete_list:
            for tuli in grease_pencil.layers:
                if(delete == tuli.info):
                    grease_pencil.layers.remove(tuli)
                    break

        sketchIdea_mesh = bpy.data.meshes[nimi]

        bpy.data.meshes.remove(sketchIdea_mesh)

        bpy.data.meshes.new(name=nimi)
        bpy.data.meshes[nimi].use_fake_user = True
        mee = bpy.data.meshes[nimi]

        bm = bmesh.new()

        for v in lista:
            bm.verts.new(v)

        bm.to_mesh(mee)
        bm.free()

        bpy.ops.object.mode_set(mode='OBJECT')
        bpy.context.view_layer.objects.active = active_object
        bpy.ops.object.mode_set(mode=active_mode)




        return {'FINISHED'}

class SculptMode(bpy.types.Operator):
    bl_idname = "object.sculpt_mode"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):

        active_object = bpy.context.view_layer.objects.active

        if (bpy.context.view_layer.objects.find('SketchingIdea') == -1):
            bpy.ops.object.gpencil_add(align='WORLD', location=(0, 0, 0), type='EMPTY')
            bpy.context.view_layer.objects[-1].name = 'SketchingIdea'
            bpy.ops.object.gpencil_add(align='WORLD', location=(0, 0, 0), type='EMPTY')
            bpy.context.view_layer.objects[-1].name = 'Rotate_Ghost'

        sculpt_name = bpy.context.scene.sketchIdea.sculpt_name

        bpy.ops.object.mode_set(mode='OBJECT')
        print('sculpt_name: ', sculpt_name)
        if (bpy.context.view_layer.objects.active.type == 'MESH'):
            sculpt_name =  bpy.context.view_layer.objects.active.name
        if(sculpt_name):
            bpy.context.view_layer.objects.active = bpy.context.view_layer.objects[sculpt_name]
        else:
            bpy.context.view_layer.objects.active = active_object
            sculpt_name = active_object.name
        bpy.ops.object.mode_set(mode='SCULPT')
        bpy.ops.sculpt.dynamic_topology_toggle()
        bpy.context.scene.sketchIdea.sculpt_name = ''

        bpy.ops.view3d.draw_op()


        return {'FINISHED'}

class SketchMode(bpy.types.Operator):
    bl_idname = "object.sketch_mode"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):

        active_object = bpy.context.view_layer.objects.active

        if (bpy.context.view_layer.objects.find('SketchingIdea') == -1):
            bpy.ops.object.gpencil_add(align='WORLD', location=(0, 0, 0), type='EMPTY')
            bpy.context.view_layer.objects[-1].name = 'SketchingIdea'
            bpy.ops.object.gpencil_add(align='WORLD', location=(0, 0, 0), type='EMPTY')
            bpy.context.view_layer.objects[-1].name = 'Rotate_Ghost'

        bpy.context.view_layer.objects.active = active_object

        if(bpy.context.view_layer.objects.active.name != 'SketchingIdea'):
            bpy.context.scene.sketchIdea.sculpt_name = bpy.context.view_layer.objects.active.name

            bpy.ops.object.mode_set(mode='OBJECT')
            bpy.context.view_layer.objects.active = bpy.data.objects['SketchingIdea']
            bpy.ops.object.mode_set(mode='PAINT_GPENCIL')
            bpy.ops.view3d.draw_op()


        return {'FINISHED'}

class ObjectMode(bpy.types.Operator):
    bl_idname = "object.object_mode"
    bl_label = "Invokes a Script"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):



        bpy.ops.object.mode_set(mode='OBJECT')


        return {'FINISHED'}

class SCENE_PT_SketchIdea(bpy.types.Panel):
    bl_label = "Sketching Idea"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = 'View'



    def draw(self, context):
        layout = self.layout

        if( bpy.context.view_layer.objects.active.type == 'MESH' or  bpy.context.view_layer.objects.active.name == 'SketchingIdea'):
            sketchIdea = bpy.context.scene.sketchIdea

            #Here you add your GUI
            row = layout.row()
            row.operator("object.showhide", text="Show/Hide Strokes")
            row = layout.row()
            row.operator("object.delete_all", text="Delete All")

            row = layout.row()
            row.operator("object.delete_visible", text="Delete Visible")

            row = layout.row()
            row.operator("object.rotate_strokes", text="Toggle Ghost")

            row = layout.row()
            row.prop(sketchIdea, "show_front_back", text="Show strokes front/back")

            row = layout.row()
            row.prop(sketchIdea, "overall_opacity", text="Overall opacity")

            row = layout.row()
            row.prop(sketchIdea, "ghost_opacity", text="Ghost opacity")



            row = layout.row()
            row.prop(sketchIdea, "symmetry_strokes", text="Apply symmetry")

            row = layout.row()
            row.prop(sketchIdea, "angle_fallout", text="Angle fallout")

            row = layout.row()
            row.prop(sketchIdea, "distance_fallout", text="Distance fallout")
            #row = layout.row()
            #row.prop(sketchIdea, "per_object_strokes", text="Strokes per object")

class SceneSketchIdea(PropertyGroup):

    angle_fallout: IntProperty(
        name="StrokeVector",
        description="StrokeVector",
        default=6, min=1, max=100
    )
    show_front_back: BoolProperty(
        name="ShowFrontBack",
        description="ShowFrontBack",
        default = False
    )
    ghosted_views: BoolProperty(
        name="ShowFrontBack",
        description="ShowFrontBack",
        default=False
    )
    hide_all: BoolProperty(
        name="ShowFrontBack",
        description="ShowFrontBack",
        default=False
    )
    distance_fallout: IntProperty(
        name="DistanceSmooth",
        description="DistanceSmooth",
        default=400, min=1, max=400
    )
    overall_opacity: FloatProperty(
        name="DistanceSmooth",
        description="DistanceSmooth",
        default=1.0, min=0, max=1
    )
    ghost_opacity: FloatProperty(
        name="GhostOpacity",
        description="DistanceSmooth",
        default=0.2, min=0, max=1
    )
    per_object_strokes: BoolProperty(
        name="ShowFrontBack",
        description="ShowFrontBack",
        default=True
    )
    sculpt_name: StringProperty(
        name="StrokeVector",
        description="StrokeVector",
        default=""
    )
    ghost: BoolProperty(
        name="ShowFrontBack",
        description="ShowFrontBack",
        default=False
    )
    symmetry_strokes: BoolProperty(
        name="ShowFrontBack",
        description="ShowFrontBack",
        default=True
    )


class VertexSketchName(PropertyGroup):

    layer_list: StringProperty(
        name="StrokeVector",
        description="StrokeVector",
        default=""
    )


class VIEW3D_PIE_MT_template(Menu):
    # label is displayed at the center of the pie menu.
    bl_label = "Sketching Idea Menu"
    bl_idname = "MES_MT_mypie"

    def draw(self, context):
        layout = self.layout

        pie = layout.menu_pie()
        # operator_enum will just spread all available options
        # for the type enum of the operator on the pie
        pie.operator("object.sculpt_mode", text='Sculpt mode')
        pie.operator("object.sketch_mode", text='Sketching mode')
        pie.operator("object.object_mode", text='Object mode')


classes = (
    SCENE_PT_SketchIdea,
    ShowHide,
    DeleteAll,
    DeleteVisible,
    OT_draw_operator,
    SceneSketchIdea,
    VertexSketchName,
    VIEW3D_PIE_MT_template,
    SketchMode,
    SculptMode,
    RotateStrokes,
    ObjectMode,
    )

def register():

    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

    bpy.types.Scene.sketchIdea = PointerProperty(type = SceneSketchIdea)
    bpy.types.Object.sketchIdea = PointerProperty(type = VertexSketchName)


    wm = bpy.context.window_manager
    km = wm.keyconfigs.addon.keymaps.new(name='Grease Pencil')
    kmi = km.keymap_items.new("wm.call_menu_pie", 'A', 'PRESS', shift=True, ctrl=True).properties.name="MES_MT_mypie"


def unregister():

    import bpy
    from bpy.utils import unregister_class

    del bpy.types.Scene.sketchIdea

    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)

    addon_keymaps.clear()

    for cls in reversed(classes):
        unregister_class(cls)

if __name__ == "__main__":
    register()
