import bpy
from bpy.types import Operator

import bgl
import blf
import numpy

import bmesh

import gpu
from gpu_extras.batch import batch_for_shader

import mathutils
import math

from mathutils import Vector
from mathutils.bvhtree import BVHTree
from mathutils.geometry import intersect_line_plane

from bpy_extras.view3d_utils import (
    region_2d_to_vector_3d,
    region_2d_to_origin_3d
)

class OT_draw_operator(Operator):
    bl_idname = "view3d.draw_op"
    bl_label = "Draw operator"
    bl_description = "Operator for drawing" 
    bl_options = {'REGISTER'}
    	
    def __init__(self):
        self.actice_object = bpy.context.active_object.name
        self.active_cursor = bpy.context.active_object.name
        self.draw_handle_2d = None
        self.draw_handle_3d = None
        self.draw_event  = None
        self.mouse_vert = None
        self.offset = 0.01
        self.mousepressed = False

        try:
            self.layercount = len(bpy.context.scene.objects['SketchingIdea'].data.layers)
        except:
            self.layercount = 0

    def execute(self, context):
        return self.invoke(context, None)

    def invoke(self, context, event):
        args = (self, context)
        self.register_handlers(args, context)

        create_new_gpencil = True

        original_type = bpy.context.area.type
        bpy.context.area.type = "VIEW_3D"
        bpy.ops.view3d.snap_cursor_to_selected()
        bpy.context.area.type = original_type

        context.window_manager.modal_handler_add(self)

        create_new_mesh = True
        find_name = "SketchingIdea"
        for find_item in bpy.data.meshes:

            if (find_item.name == find_name):
                create_new_mesh = False
                break

        if(create_new_mesh):
            nimi = "SketchingIdea"
            bpy.data.meshes.new(nimi)
            bpy.data.meshes[nimi].use_fake_user = True


        return {"RUNNING_MODAL"}

    def register_handlers(self, args, context):

        self.draw_handle_2d = bpy.types.SpaceView3D.draw_handler_add(
            self.draw_callback_2d, args, "WINDOW", "POST_PIXEL")

        self.draw_event = context.window_manager.event_timer_add(0.1, window=context.window)
        
    def unregister_handlers(self, context):

        context.window_manager.event_timer_remove(self.draw_event)
        bpy.types.SpaceView3D.draw_handler_remove(self.draw_handle_2d, "WINDOW")

        self.draw_handle_2d = None
        self.draw_event  = None
        self.bvhtree = None

    def get_origin_and_direction(self, event, context):
        region    = context.region
        region_3d = context.space_data.region_3d
        mouse_coord = (event.mouse_region_x, event.mouse_region_y)

        origin    = region_2d_to_origin_3d(region, region_3d, mouse_coord)
        direction = region_2d_to_vector_3d(region, region_3d, mouse_coord)

        return origin, direction


    def get_mouse_3d_on_mesh(self, event, context):

        origin, direction = self.get_origin_and_direction(event, context)

        self.hit, self.normal, *_ = self.bvhtree.ray_cast(origin, direction)
        if self.hit is not None:
            self.hit = self.hit + (self.normal * self.offset)

        return self.hit

    def get_mouse_3d_on_plane(self, event, context):

        origin, direction = self.get_origin_and_direction(event, context)

        # get the intersection point on infinite plane
        return intersect_line_plane(origin, origin + direction,
        self.hit, self.normal)
        
            
    def modal(self, context, event):
        if context.area:
            context.area.tag_redraw()
                               
        if event.type in {"ESC"}:
            self.unregister_handlers(context)
            return {'CANCELLED'}

        if event.value == "NOTHING":



            if bpy.context.active_object != None:
                if(self.active_cursor != bpy.context.active_object.name):
                    original_type = bpy.context.area.type
                    bpy.context.area.type = "VIEW_3D"
                    bpy.ops.view3d.snap_cursor_to_selected()
                    bpy.context.area.type = original_type
                    self.active_cursor = bpy.context.active_object.name



                y = getattr(bpy.context.scene.objects['SketchingIdea'].data, "layers", None)
                value = True
                if y is not None:
                    try:
                        bpy.context.scene.objects['SketchingIdea'].data.layers
                    except AttributeError:
                        value = False

                    if (value):
                        if(len(bpy.context.scene.objects['SketchingIdea'].data.layers) > 0):

                            angle_fallout = bpy.context.scene.sketchIdea.angle_fallout
                            show_front_back = bpy.context.scene.sketchIdea.show_front_back
                            distance_fallout = bpy.context.scene.sketchIdea.distance_fallout
                            overall_opacity = bpy.context.scene.sketchIdea.overall_opacity
                            ghost_opacity = bpy.context.scene.sketchIdea.ghost_opacity

                            try:
                                origin, direction = self.get_origin_and_direction(event, context)
                            except AttributeError:
                                self.unregister_handlers(context)
                                return {'CANCELLED'}
                            nimi = 'SketchingIdea'

                            for index, vertice in enumerate(bpy.data.meshes[nimi].vertices):
                                camera_norm = numpy.array([origin[0], origin[1], origin[2]])
                                camera_norm = numpy.linalg.norm(camera_norm)
                                origin_unit = (origin[0] / camera_norm, origin[1] / camera_norm, origin[2] / camera_norm)

                                stroke_norm = numpy.array([vertice.co[0], vertice.co[1], vertice.co[2]])
                                stroke_norm = numpy.linalg.norm(stroke_norm)
                                stroke_unit = (vertice.co[0] / stroke_norm, vertice.co[1] / stroke_norm, vertice.co[2] / stroke_norm)
                                final_vector = numpy.dot(origin_unit, stroke_unit, None)

                                if(camera_norm > 100):
                                    camera_norm = 100
                                if(stroke_norm > 100):
                                    stroke_norm = 100
                                diff_length = abs(camera_norm-stroke_norm)
                                fallout_index = diff_length/distance_fallout
                                if(fallout_index > 1):
                                    fallout_index = 1
                                final_distance_index = 1 - fallout_index


                                if(show_front_back):
                                    if(angle_fallout % 2 == 1):
                                        final = pow(final_vector, angle_fallout + 1)
                                        if(bpy.context.scene.objects['SketchingIdea'].data.layers[index].parent != None):
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[index].opacity = ghost_opacity
                                        else:
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[index].opacity = final * final_distance_index *  overall_opacity
                                    else:
                                        final = pow(final_vector, angle_fallout)
                                        if (bpy.context.scene.objects['SketchingIdea'].data.layers[
                                            index].parent  != None):
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[
                                                index].opacity = ghost_opacity
                                        else:
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[
                                                index].opacity = final * final_distance_index * overall_opacity
                                else:
                                    if (angle_fallout % 2 == 0):
                                        final = pow(final_vector, angle_fallout + 1)
                                        if (bpy.context.scene.objects['SketchingIdea'].data.layers[
                                            index].parent != None):
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[
                                                index].opacity = ghost_opacity
                                        else:
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[
                                                index].opacity = final * final_distance_index * overall_opacity
                                    else:
                                        final = pow(final_vector, angle_fallout)
                                        if (bpy.context.scene.objects['SketchingIdea'].data.layers[
                                            index].parent != None):
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[
                                                index].opacity = ghost_opacity
                                        else:
                                            bpy.context.scene.objects['SketchingIdea'].data.layers[
                                                index].opacity = final * final_distance_index * overall_opacity



        if event.value == "RELEASE":
            # Left mouse button pressed
            symmetry_strokes = bpy.context.scene.sketchIdea.symmetry_strokes

            y = getattr(bpy.context.scene.objects['SketchingIdea'].data, "layers", None)
            layer = True
            value = True
            testi = ""
            if y is not None:
                try:
                    bpy.context.scene.objects['SketchingIdea'].data.layers
                except AttributeError:
                    layer = False

                if(layer):
                    if(len(bpy.context.scene.objects['SketchingIdea'].data.layers) > 0):
                        try:
                            bpy.context.scene.objects['SketchingIdea'].data.layers[-1].active_frame.strokes
                        except AttributeError:
                            value = False

                        if(value):

                            origin, direction = self.get_origin_and_direction(event, context)

                            symmetry_axis = (0,0,1)
                            symmetry_axis2 = (1, 0, 1)
                            camera_norm = numpy.array([origin[0], origin[1], origin[2]])
                            camera_norm = numpy.linalg.norm(camera_norm)
                            origin_unit = (origin[0] / camera_norm, origin[1] / camera_norm, origin[2] / camera_norm)
                            final_vector = numpy.dot(origin_unit, symmetry_axis, None)
                            final_vector2 = numpy.dot(origin_unit, symmetry_axis2, None)
                            cursor_loc = bpy.context.scene.cursor.location[0]

                            if((symmetry_strokes and ((final_vector > -0.35 and final_vector2 > -0.35) and (final_vector < 0.35 and final_vector2 < 0.35))) or (symmetry_strokes and final_vector > 0.8 and final_vector2 > 0.8 )):
                                frame = bpy.context.scene.objects['SketchingIdea'].data.layers[-1].active_frame
                                frame.strokes.new()
                                new_stroke = frame.strokes[-1]
                                new_stroke.points.add(len(frame.strokes[0].points), pressure=1, strength=1)
                                for index, point in enumerate(frame.strokes[0].points):
                                    new_x = cursor_loc-point.co[0]
                                    new_xx = cursor_loc + new_x
                                    new_stroke.points[index].co = (new_xx, point.co[1], point.co[2])
                                    new_stroke.points[index].pressure = point.pressure
                                    new_stroke.points[index].strength = point.strength
                                    new_stroke.line_width = frame.strokes[0].line_width
                                new_stroke.display_mode = '3DSPACE'


                            nimi = "SketchingIdea"
                            me = bpy.data.meshes[nimi]

                            bm = bmesh.new()
                            bm.from_mesh(me)
                            v = (origin[0], origin[1], origin[2])
                            bm.verts.new(v)

                            bm.to_mesh(me)

                            bm.to_mesh(bpy.data.meshes[nimi])
                            bm.free()
                            bpy.context.scene.objects['SketchingIdea'].data.layers.new(name="Sketch")
                            self.layercount += 1




        return {"PASS_THROUGH"}


	# Draw handler to paint in pixels
    def draw_callback_2d(self, op, context):
        # Draw text to indicate that draw mode is active
        region = context.region
        text = "- Sketching Idea -"
        subtext = "Esc : Close"

        xt = int(region.width / 2.0)
        
        blf.size(0, 24, 72)
        blf.position(0, xt - blf.dimensions(0, text)[0] / 2, 60 , 0)
        blf.draw(0, text) 

        blf.size(1, 20, 72)
        blf.position(1, xt - blf.dimensions(0, subtext)[0] / 2, 30 , 1)
        blf.draw(1, subtext) 

